@@ -1,4 +1,5 @@
 import codecs
+import calendar
 import datetime
 import hashlib
 import hmac
@@ -110,6 +111,7 @@ class AdsAPI(object):
         try:
             if method == 'GET':
                 url = '%s/%s?%s' % (FACEBOOK_API, path, urllib.urlencode(args))
+                print url
                 f = urllib2.urlopen(url)
             elif method == 'POST':
                 url = '%s/%s' % (FACEBOOK_API, path)
@@ -291,14 +293,20 @@ class AdsAPI(object):
             args = {'hashes': hashes}
         return self.make_request(path, 'GET', args, batch=batch)
 
-    def get_stats_by_adaccount(self, account_id, batch=False):
+    def get_stats_by_adaccount(self, account_id, batch=False, start_time=None, end_time=None):
         """Returns the stats for a Facebook campaign."""
         path = 'act_%s/adcampaignstats' % account_id
-        return self.make_request(path, 'GET', batch=batch)
+        args = {}
+        if start_time:
+            args['start_time'] = self.__parse_time(start_time)
+        if end_time:
+            args['end_time'] = self.__parse_time(end_time)
+        return self.make_request(path, 'GET', args, batch=batch)
 
     # New API
     def get_stats_by_adcampaign_group(
-            self, campaign_group_id, fields=None, filters=None, batch=False):
+            self, campaign_group_id, fields=None, filters=None, batch=False,
+            start_time=None, end_time=None):
         """Returns the stats for a Facebook campaign group."""
         path = '%s/stats' % campaign_group_id
         args = {}
@@ -306,23 +314,37 @@ class AdsAPI(object):
             args['fields'] = json.dumps(fields)
         if filters:
             args['filters'] = json.dumps(filters)
+        if start_time:
+            args['start_time'] = self.__parse_time(start_time)
+        if end_time:
+            args['start_time'] = self.__parse_time(end_time)
         return self.make_request(path, 'GET', args, batch=batch)
 
     def get_stats_by_adcampaign(self, account_id, campaign_ids=None,
-                                batch=False):
+                                batch=False, start_time=None, end_time=None):
         """Returns the stats for a Facebook campaign by adcampaign."""
         path = 'act_%s/adcampaignstats' % account_id
         args = {}
         if campaign_ids is not None:
             args['campaign_ids'] = json.dumps(campaign_ids)
+        if start_time:
+            args['start_time'] = self.__parse_time(start_time)
+        if end_time:
+            args['start_time'] = self.__parse_time(end_time)
         return self.make_request(path, 'GET', args, batch=batch)
 
-    def get_stats_by_adgroup(self, account_id, adgroup_ids=None, batch=False):
+    def get_stats_by_adgroup(
+            self, account_id, adgroup_ids=None, batch=False,
+            start_time=None, end_time=None):
         """Returns the stats for a Facebook campaign by adgroup."""
         path = 'act_%s/adgroupstats' % account_id
         args = {}
         if adgroup_ids is not None:
             args['adgroup_ids'] = json.dumps(adgroup_ids)
+        if start_time:
+            args['start_time'] = self.__parse_time(start_time)
+        if end_time:
+            args['start_time'] = self.__parse_time(end_time)
         return self.make_request(path, 'GET', args, batch=batch)
 
     # New API
@@ -380,7 +402,7 @@ class AdsAPI(object):
         if filters:
             args['filters'] = json.dumps(filters)
         if actions_group_by:
-            args['actions_group_by'] = actions_group_by
+            args['actions_group_by'] = json.dumps(actions_group_by)
         if async:
             args['async'] = 'true'
             return self.make_request(path, 'POST', args=args, batch=batch)
@@ -433,7 +455,7 @@ class AdsAPI(object):
         path = '%s/conversions' % adgroup_id
         return self.make_request(path, 'GET', batch=batch)
 
-    def get_custom_audiences(self, account_id, audience_id, batch=False):
+    def get_custom_audiences(self, account_id, batch=False):
         """Returns the information for a given audience."""
         path = 'act_%s/customaudiences' % account_id
         return self.make_request(path, 'GET', batch=batch)
@@ -748,7 +770,7 @@ class AdsAPI(object):
         return self.make_request(path, 'POST', args, batch=batch)
 
     def create_adgroup(self, account_id, name, bid_type, bid_info, campaign_id,
-                       creative_id, targeting, conversion_specs=None,
+                       creative_id, targeting, max_bid=None, conversion_specs=None,
                        tracking_specs=None, view_tags=None, objective=None,
                        adgroup_status=None, batch=False):
         """Creates an adgroup in the given ad camapaign with the given spec."""
@@ -761,6 +783,10 @@ class AdsAPI(object):
             'creative': json.dumps({'creative_id': creative_id}),
             'targeting': json.dumps(targeting),
         }
+        if max_bid:
+            assert bid_type == 'CPM', 'can only use max_bid with CPM bidding'
+            args['max_bid'] = max_bid
+            del args['bid_info']  # get rid of bid_info
         if conversion_specs:
             args['conversion_specs'] = json.dumps(conversion_specs)
         if tracking_specs:
@@ -853,3 +879,16 @@ class AdsAPI(object):
             'tag': tag,
         }
         return self.make_request(path, 'POST', args, batch=batch)
+
+    def __parse_time(self, time_obj):
+        """Internal function to transform user supplied time objects into Unix time."""
+        if time_obj:
+            resp = ''
+            if isinstance(time_obj, int) or isinstance(time_obj, str):
+                resp = time_obj
+            elif isinstance(time_obj, datetime.datetime):
+                resp = calendar.timegm(time_obj.timetuple())
+            else:
+                raise Exception("Unknown __parse_time format for {0}".format(time_obj))
+            return str(resp)
+        return None
